local _G = _G
local AVR = AVR
local math_rad = math.rad
local math_cos = math.cos
local math_sin = math.sin
local math_deg = math.deg
local GetPlayerFacing = GetPlayerFacing
local unpack = unpack
local pairs = pairs

-- Isolate the environment
local folder, core = ...
setfenv(1, core)
-- local

fireNovaRange = 10 --Fire Nova has a range of 10 yards.

scene = {} --Our AVR scene.

local prev_OnEnable = core.OnEnable
function core:OnEnable()
	prev_OnEnable(self)
	AVR = _G.AVR
	if AVR then
		AddAVROptions()
		
		scene = AVR:GetTempScene(folder)
		scene:ClearScene()
		scene.visible = true
		
	end
end

local prev_OnDisable = core.OnDisable
function core:OnDisable()
	prev_OnDisable(self)
	
	for totemGUID in pairs(totemMeshes) do 
		RemoveTotemFromAVR(totemGUID)
	end
	scene:ClearScene()
	scene.visible = false
end
	
totemMeshes = {}
fireTotemRange = {}
function AddTotemToAVR(totemGUID, x, y, totemSlot, spellID, range, precise)
	if AVR then
		if totemMeshes[totemGUID] then --it's already in our list? odd
			RemoveTotemFromAVR(totemGUID)
		end
		
		totemMeshes[totemGUID] = totemMeshes[totemGUID] or {}
		
		local bX, bY = GetAVRYardCoords(x, y)
		local range = range or totemInfo[spellID].range or 3
		
		local m
		
		m = AVRCircleMesh:New(range, 30, false, P.AVRLineThickness * 32)
		m:TranslateMesh(bX, bY)
		m:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, 1)
		scene:AddMesh(m)
		totemMeshes[totemGUID].rangeRing = m

		
		--[[ Totem coloured ring sits just inside range ring.]]
		if P.AVRTotemColourRing == true then
			m = AVRCircleMesh:New(range-0.2, 30, true, P.AVRLineThickness * 32)
			m:TranslateMesh(bX, bY)
			m:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, 1) --P.totemColour[totemSlot].a
			scene:AddMesh(m)
			totemMeshes[totemGUID].colourRing = m
		end
		
		--[[ Filled in circle coloured to the totem colour. off by default.
		if P.AVRSolidCircle == true then
			m = AVRFilledCircleMesh:New(range or 3,30, false)
			m:TranslateMesh(bX, bY)
			m:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, (-0.0033333333*range+0.18333333))
			scene:AddMesh(m)
			totemMeshes[totemGUID].solidCircle = m
		end]]
			
		--[[ Small texture under totem.]]
		if P.AVRTotemLocation == true then
			m = AVRFilledCircleMesh:New(1,6, false, nil)
			m:TranslateMesh(bX, bY)
			m:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, .2)
			scene:AddMesh(m)
			totemMeshes[totemGUID].smallCircle = m
		end
		
		--[[ Add a fire nova ring if it's a fire totem.]]
		if totemSlot == 1 and P.avrFireNovaRing == true then
			if precise == true then
				range = fireNovaRange --We know the exact location of the totem, set ring at 10 yards.
			else
				range = fireNovaRange - spawnDistance --Don't know the totem's location, set ring at 7 yards.
			end
			
			m = AVRCircleMesh:New(range, 30, false, P.AVRLineThickness * 32)
			m:TranslateMesh(bX, bY)
			m:SetColor(P.avrFireNovaColour.r, P.avrFireNovaColour.g, P.avrFireNovaColour.b, 1)
			scene:AddMesh(m)
			totemMeshes[totemGUID].firenovaRangeRing = m
--~ 			totemMeshes[totemGUID].firenovaRangeRingRange = range
			fireTotemRange[totemGUID] = range
			
			
			if P.AVRTotemColourRing == true then
				m = AVRCircleMesh:New(range-0.2, 30, true, P.AVRLineThickness * 32)
				m:TranslateMesh(bX, bY)
				m:SetColor(P.avrFireNovaColour.r, P.avrFireNovaColour.g, P.avrFireNovaColour.b, 1) --P.totemColour[totemSlot].a
				scene:AddMesh(m)
				totemMeshes[totemGUID].firenovaColourRing = m
			end
		end
		
		
		
	end
end

--[[

avrRangeRing = {}
avrTotemRing = {}
avrFireNovaRangeRing = {}
avrFireNovaColourRing = {}
function AddTotemToAVR(totemGUID, x, y, totemSlot, spellID, range, precise)--	/run TR.AddTotemToAVR()
	if AVR then
		if scene[totemGUID] then
			RemoveTotemFromAVR(totemGUID)
		end
		
		if AVR.AddCircleWarningEveryone then		
			scene[totemGUID] = AVR:GetTempScene("TR_"..totemGUID)
			scene[totemGUID]:ClearScene()
			scene[totemGUID].visible = true
			
			local bX, bY = GetAVRYardCoords(x, y)
	--~ 		Debug("avr07","bX, bY", bX, bY)
			local range = range or totemInfo[spellID].range or 3
			
			local m = AVRCircleMesh:New(range, 30, false, P.AVRLineThickness * 32)
			m:TranslateMesh(bX, bY)
			m:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, 1)
			scene[totemGUID]:AddMesh(m)
			avrRangeRing[totemGUID] = m
			
			if P.AVRTotemColourRing == true then
				local m = AVRCircleMesh:New(range-0.2, 30, true, P.AVRLineThickness * 32)
				m:TranslateMesh(bX, bY)
				m:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, 1) --P.totemColour[totemSlot].a
				scene[totemGUID]:AddMesh(m)
				avrTotemRing[totemGUID] = m
			end

			if P.AVRSolidCircle == true then
				local m = AVRFilledCircleMesh:New(range or 3,30, false)
				m:TranslateMesh(bX, bY)
				m:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, (-0.0033333333*range+0.18333333))
				--(-0.005*range+0.2)) --10y = 15% alpha, 30y = .05% alpha.
				
				scene[totemGUID]:AddMesh(m)
			end
			

			if P.AVRTotemLocation == true then
				local m = AVRFilledCircleMesh:New(1,6, false, nil)
				m:TranslateMesh(bX, bY)
				m:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, .2)
				scene[totemGUID]:AddMesh(m)
			end

			if totemSlot == 1 and P.avrFireNovaRing == true then
				if precise == true then
					range = 10
				else
					range = range - spawnDistance
				end
				
				local m = AVRCircleMesh:New(range, 30, false, P.AVRLineThickness * 32)
				m:TranslateMesh(bX, bY)
				m:SetColor(P.avrFireNovaColour.r, P.avrFireNovaColour.g, P.avrFireNovaColour.b, 1)
				scene[totemGUID]:AddMesh(m)
				avrFireNovaRangeRing[totemGUID] = avrFireNovaRangeRing[totemGUID] or {}
				avrFireNovaRangeRing[totemGUID].mesh = m
				avrFireNovaRangeRing[totemGUID].range = range
				
				
				if P.AVRTotemColourRing == true then
					local m = AVRCircleMesh:New(range-0.2, 30, true, P.AVRLineThickness * 32)
					m:TranslateMesh(bX, bY)
					m:SetColor(P.avrFireNovaColour.r, P.avrFireNovaColour.g, P.avrFireNovaColour.b, 1) --P.totemColour[totemSlot].a
					scene[totemGUID]:AddMesh(m)
					
					avrFireNovaColourRing[totemGUID] = m
					
				end
			end
			
		end
	end
end


function RemoveTotemFromAVR(totemGUID)--	/run TR.RemoveTotemFromAVR()
	if AVR and scene[totemGUID] then
		scene[totemGUID]:ClearScene()
		scene[totemGUID].visible = false
		scene[totemGUID] = nil
	end
end
]]

function RemoveTotemFromAVR(totemGUID)
	if AVR and totemMeshes[totemGUID] then
		for name, mesh in pairs(totemMeshes[totemGUID]) do 
			mesh:Remove()
			mesh = nil
		end
		totemMeshes[totemGUID] = nil
	end
end


--------------------------------------------------
function GetAVRYardCoords(aX, aY)				--
-- AVR converts numbers differently then I do.	--
-- Thier X is the same, but Y is not. 			--
--------------------------------------------------
	local sX, sY = unpack(AVR.zoneData.currentScale)
	return aX*sX,(1-aY)*sY
end

